""" Embeds a secret message into an image as follows:
    1) Assignments each character to a character code:
    * the letters 'A' to 'Z' have character code 1 to 26,
    * the letters 'a' to 'z' also use codes 1 to 26,
    * the ' ' (space) character has character code 0,
    * the '.' (period) character has character code 27, and
    * the '?' (question mark) character has character code 28.
    * all other characters have character code 29.

    2) Starting at pixel x=0 and y=0 and row-wise across the picture
    the RGB values are adjusted to embed the sequence of characters
    in the message.

    3) To make as small of a color change to a pixel the character
    code's value is split evenly across each R, G, and B. For example,
    to embed the letter 'j' with character code of 10, we:
    a) use integer division of 10 / 3 to get 3,
    b) adjust R by 3 and G by 3,
    c) adjust B by the rest, i.e., (10 - 3 - 3) or 4
    NOTE:  to avoid going outside the range of values 0 - 255 when
    adjusting the RGB values, values below 128 will be adjusted upward
    and values above or equal to 128 will be adjusted downward.
"""
def main():
  print "Select the Media Folder"
  setMediaFolder()
  print "Select the picture (.jpg) file in which to embed the secret message"
  fileName = pickAFile()
  pict = makePicture(fileName)
  show(pict)
  
  print "Select the text file (.txt) file containing the secret message"
  msgFileName = pickAFile()

  print "Please wait while picture is processed."
  pictWithMsg = embedMsg(pict, msgFileName)
  
  show(pictWithMsg)
  openPictureTool(pict)
  openPictureTool(pictWithMsg)
  newFileName = getMediaPath('embeddedMsgPicture.png')
  writePictureTo(pictWithMsg, newFileName)


def embedMsg(picture, msgFileName):
  """ Embeds the secret message contained in the msgFileName into the picture
      and returns the picture containing the embedded secret message """
  targetPict = makeEmptyPicture(getWidth(picture), getHeight(picture))
  print "msgFileName", msgFileName
  msgFile = open(msgFileName,'r')
  #msgFile = open(getMediaPath(msgFileName),'r')
  wholeMsgString = msgFile.read()   # reads whole file as a single string
  msgIndex = 0
  msgLength = len(wholeMsgString)
  for y in range(0 , getHeight(picture)):
    for x in range(0 , getWidth(picture)):
      pixel = getPixel(picture,x,y)
      targetPixel = getPixel(targetPict,x,y)
      if msgIndex < msgLength:
        charCode = determineCharCode(wholeMsgString[msgIndex])
        r = getRed(pixel)
        g = getGreen(pixel)
        b = getBlue(pixel)
        if r < 128:
          redAdjustment = charCode / 3
        else:
          redAdjustment = - (charCode / 3)

        if g < 128:
          greenAdjustment = charCode / 3
        else:
          greenAdjustment = - (charCode / 3)

        if b < 128:
          blueAdjustment = charCode - abs(redAdjustment) - abs(greenAdjustment)
        else:
          blueAdjustment = - (charCode - abs(redAdjustment) - abs(greenAdjustment))
        # print wholeMsgString[msgIndex],"charCode",charCode,"Adjustments",redAdjustment,greenAdjustment, blueAdjustment
        targetPixelColor = makeColor(r+redAdjustment, g+greenAdjustment, b+blueAdjustment)
      else:  
        # done embedding message so just copy pixel
        targetPixelColor = getColor(pixel)

      msgIndex = msgIndex + 1

      setColor(targetPixel, targetPixelColor)
  return targetPict
  
def determineCharCode(msgChar):
  """ Returns the numeric character code for the character msgChar """
  msgChar = msgChar.upper()
  if msgChar >= 'A' and msgChar <= 'Z':
    return ord(msgChar) - ord('A') + 1
  elif msgChar == '.':
    return 27
  elif msgChar == '?':
    return 28
  elif msgChar == ' ':  #space character
    return 0
  else:
    return 29  # other character
              
main() # starts the program
